@@ -28,9 +28,15 @@
 #include "ot-main.h"
 
 gboolean verbose;
+gboolean opt_prefer_loose;
+gboolean opt_related;
+gint opt_depth;
 
 static GOptionEntry options[] = {
   { "verbose", 'v', 0, G_OPTION_ARG_NONE, &verbose, "Show more information", NULL },
+  { "prefer-loose", 0, 0, G_OPTION_ARG_NONE, &opt_prefer_loose, "Download loose objects by default", NULL },
+  { "related", 0, 0, G_OPTION_ARG_NONE, &opt_related, "Download related commits", NULL },
+  { "depth", 0, 0, G_OPTION_ARG_INT, &opt_depth, "Download parent commits up to this depth (default: 0)", NULL },
   { NULL },
 };
 
@@ -338,7 +344,7 @@ find_object_in_remote_packs (OtPullData       *pull_data,
                              GError          **error)
 {
   gboolean ret = FALSE;
-  guint64 ret_offset;
+  guint64 ret_offset = 0;
   guint i;
   GPtrArray *iter;
   ot_lvariant GVariant *csum_bytes_v = NULL;
@@ -664,6 +670,7 @@ fetch_and_store_metadata (OtPullData          *pull_data,
 
 static gboolean
 fetch_and_store_tree_metadata_recurse (OtPullData   *pull_data,
+                                       int           depth,
                                        const char   *rev,
                                        GCancellable *cancellable,
                                        GError      **error)
@@ -676,6 +683,13 @@ fetch_and_store_tree_metadata_recurse (OtPullData   *pull_data,
   ot_lobj GFile *stored_path = NULL;
   ot_lfree char *pack_checksum = NULL;
 
+  if (depth > OSTREE_MAX_RECURSION)
+    {
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                   "Exceeded maximum recursion");
+      goto out;
+    }
+
   if (!fetch_and_store_metadata (pull_data, rev, OSTREE_OBJECT_TYPE_DIR_TREE,
                                  &tree, cancellable, error))
     goto out;
@@ -724,7 +738,7 @@ fetch_and_store_tree_metadata_recurse (OtPullData   *pull_data,
 
       g_free (tmp_checksum);
       tmp_checksum = ostree_checksum_from_bytes_v (tree_csum);
-      if (!fetch_and_store_tree_metadata_recurse (pull_data, tmp_checksum, cancellable, error))
+      if (!fetch_and_store_tree_metadata_recurse (pull_data, depth+1, tmp_checksum, cancellable, error))
         goto out;
     }
 
@@ -735,15 +749,19 @@ fetch_and_store_tree_metadata_recurse (OtPullData   *pull_data,
 
 static gboolean
 fetch_and_store_commit_metadata_recurse (OtPullData   *pull_data,
+                                         int           parent_depth,
+                                         int           related_depth,
                                          const char   *rev,
                                          GCancellable *cancellable,
                                          GError      **error)
 {
   gboolean ret = FALSE;
   ot_lvariant GVariant *commit = NULL;
+  ot_lvariant GVariant *related_objects = NULL;
   ot_lvariant GVariant *tree_contents_csum = NULL;
   ot_lvariant GVariant *tree_meta_csum = NULL;
   ot_lfree char *tmp_checksum = NULL;
+  GVariantIter *iter = NULL;
 
   if (!fetch_and_store_metadata (pull_data, rev, OSTREE_OBJECT_TYPE_COMMIT,
                                  &commit, cancellable, error))
@@ -761,12 +779,61 @@ fetch_and_store_commit_metadata_recurse (OtPullData   *pull_data,
   
   g_free (tmp_checksum);
   tmp_checksum = ostree_checksum_from_bytes_v (tree_contents_csum);
-  if (!fetch_and_store_tree_metadata_recurse (pull_data, tmp_checksum,
+  if (!fetch_and_store_tree_metadata_recurse (pull_data, 0, tmp_checksum,
                                               cancellable, error))
     goto out;
 
+  if (opt_related)
+    {
+      const char *name;
+      ot_lvariant GVariant *csum_v = NULL;
+
+      if (parent_depth > OSTREE_MAX_RECURSION
+          || related_depth > OSTREE_MAX_RECURSION)
+        {
+          g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
+                       "Exceeded maximum recursion");
+          goto out;
+        }
+
+      related_objects = g_variant_get_child_value (commit, 2);
+      iter = g_variant_iter_new (related_objects);
+
+      while (g_variant_iter_loop (iter, "(&s@ay)", &name, &csum_v))
+        {
+          ot_lfree char *checksum = ostree_checksum_from_bytes_v (csum_v);
+
+          /* Pass opt_depth here to ensure we aren't fetching parents of related */
+          if (!fetch_and_store_commit_metadata_recurse (pull_data, opt_depth,
+                                                        related_depth + 1, checksum,
+                                                        cancellable, error))
+            goto out;
+        }
+    }
+
+  if (parent_depth < opt_depth)
+    {
+      ot_lvariant GVariant *parent_csum_v = NULL;
+
+      parent_csum_v = g_variant_get_child_value (commit, 1);
+
+      if (g_variant_n_children (parent_csum_v) > 0)
+        {
+          ot_lfree char *checksum = ostree_checksum_from_bytes_v (parent_csum_v);
+
+          g_printerr ("Acquring parent %s of %s\n", checksum, rev);
+
+          if (!fetch_and_store_commit_metadata_recurse (pull_data, parent_depth + 1,
+                                                        0, checksum,
+                                                        cancellable, error))
+            goto out;
+        }
+    }
+
   ret = TRUE;
  out:
+  if (iter)
+    g_variant_iter_free (iter);
   return ret;
 }
 
@@ -891,7 +958,7 @@ fetch_content (OtPullData           *pull_data,
                                        cancellable, error))
         goto out;
 
-      if (remote_pack_checksum)
+      if (remote_pack_checksum && !opt_prefer_loose)
         {
           files_to_fetch = g_hash_table_lookup (data_packs_to_fetch, remote_pack_checksum);
           if (files_to_fetch == NULL)
@@ -966,8 +1033,8 @@ fetch_content (OtPullData           *pull_data,
                                cancellable, error))
         goto out;
 
-      if (!ot_util_variant_map (temp_path, OSTREE_FILE_HEADER_GVARIANT_FORMAT, &file_meta,
-                                error))
+      if (!ot_util_variant_map (temp_path, OSTREE_FILE_HEADER_GVARIANT_FORMAT, FALSE,
+                                &file_meta, error))
         goto out;
 
       if (!ostree_file_header_parse (file_meta, &file_info, &xattrs, error))
@@ -992,9 +1059,9 @@ fetch_content (OtPullData           *pull_data,
                                               cancellable, error))
         goto out;
 
-      if (!ostree_repo_stage_object (pull_data->repo, OSTREE_OBJECT_TYPE_FILE, checksum,
-                                     file_object_input,
-                                     cancellable, error))
+      if (!ostree_repo_stage_file_object (pull_data->repo, checksum,
+                                          file_object_input, length,
+                                          cancellable, error))
         goto out;
     }
 
@@ -1062,7 +1129,46 @@ parse_ref_summary (const char    *contents,
   g_strfreev (lines);
   return ret;
 }
-                      
+
+static gboolean
+repo_get_string_key_inherit (OstreeRepo          *repo,
+                             const char          *section,
+                             const char          *key,
+                             char               **out_value,
+                             GError             **error)
+{
+  gboolean ret = FALSE;
+  GError *temp_error = NULL;
+  GKeyFile *config;
+  ot_lfree char *ret_value = NULL;
+
+  config = ostree_repo_get_config (repo);
+
+  ret_value = g_key_file_get_value (config, section, key, &temp_error);
+  if (temp_error)
+    {
+      OstreeRepo *parent = ostree_repo_get_parent (repo);
+      if (parent &&
+          (g_error_matches (temp_error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_KEY_NOT_FOUND)
+           || g_error_matches (temp_error, G_KEY_FILE_ERROR, G_KEY_FILE_ERROR_GROUP_NOT_FOUND)))
+        {
+          g_clear_error (&temp_error);
+          if (!repo_get_string_key_inherit (parent, section, key, &ret_value, error))
+            goto out;
+        }
+      else
+        {
+          g_propagate_error (error, temp_error);
+          goto out;
+        }
+    }
+
+  ret = TRUE;
+  ot_transfer_out_value (out_value, &ret_value);
+ out:
+  return ret;
+}
+
 static gboolean
 ostree_builtin_pull (int argc, char **argv, GFile *repo_path, GError **error)
 {
@@ -1087,6 +1193,8 @@ ostree_builtin_pull (int argc, char **argv, GFile *repo_path, GError **error)
   GKeyFile *config = NULL;
   char **configured_branches = NULL;
 
+  memset (pull_data, 0, sizeof (*pull_data));
+
   context = g_option_context_new ("REMOTE [BRANCH...] - Download data from remote repository");
   g_option_context_add_main_entries (context, options, NULL);
 
@@ -1097,7 +1205,6 @@ ostree_builtin_pull (int argc, char **argv, GFile *repo_path, GError **error)
   if (!ostree_repo_check (repo, error))
     goto out;
 
-  memset (pull_data, 0, sizeof (*pull_data));
   pull_data->repo = repo;
   pull_data->file_checksums_to_fetch = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 
@@ -1117,8 +1224,7 @@ ostree_builtin_pull (int argc, char **argv, GFile *repo_path, GError **error)
   config = ostree_repo_get_config (repo);
 
   remote_key = g_strdup_printf ("remote \"%s\"", pull_data->remote_name);
-  baseurl = g_key_file_get_string (config, remote_key, "url", error);
-  if (!baseurl)
+  if (!repo_get_string_key_inherit (repo, remote_key, "url", &baseurl, error))
     goto out;
   pull_data->base_uri = soup_uri_new (baseurl);
 
@@ -1217,7 +1323,8 @@ ostree_builtin_pull (int argc, char **argv, GFile *repo_path, GError **error)
     {
       const char *commit = value;
       
-      if (!fetch_and_store_commit_metadata_recurse (pull_data, commit, cancellable, error))
+      if (!fetch_and_store_commit_metadata_recurse (pull_data, 0, 0, commit,
+                                                    cancellable, error))
         goto out;
     }
 
@@ -1245,7 +1352,7 @@ ostree_builtin_pull (int argc, char **argv, GFile *repo_path, GError **error)
           if (!ostree_validate_checksum_string (sha256, error))
             goto out;
 
-          if (!fetch_and_store_commit_metadata_recurse (pull_data, sha256, cancellable, error))
+          if (!fetch_and_store_commit_metadata_recurse (pull_data, 0, 0, sha256, cancellable, error))
             goto out;
          
           g_hash_table_insert (updated_refs, g_strdup (ref), g_strdup (sha256));
